# -*- coding: utf-8 -*-
"""djangocms-helper - Helpers for django CMS plugin development"""
from __future__ import absolute_import, print_function, unicode_literals

__version__ = '0.9.0'
__author__ = 'Iacopo Spalletti <i.spalletti@nephila.it>'
__all__ = ['runner']

HELPER_FILE = 'cms_helper.py'
